# Working locally

The first exercises will teach you to use git locally.
This means that everything is going to be stored in the local repository (i.e. the `.git` folder).
We also repeat that you not delete that folder, or you'll lose your (local) repository.

We strongly suggest that:
- you to print the status after each command (`git status`), to be sure to understand in what state you put your files and repository,
- to print the log after each commit to see the evolution of your commits.

# Exercise 1: initialize your local repository

Let's initialize the repository.
First create a directory that will contain your work and repository, that is to say the working directory.

For example:
```
$ mkdir git-workshop-local
```
and change into your newly created directory:
```
$ cd git-workshop-local
```

Now you can initialize your repository with `git init`:
```
$ git init
```
Git will output a message stating that the repository is initialized.
Confirm that there is a `.git` subdirectory in your working directory.

# Exercise 2: add new files and do your first commit

Start by adding a new file in your working directory.
You can create a file named after your name `alfred.escher.txt` which contains your full name:
```
Alfred Escher
```
if your first and last name are respectively Alfred and Escher.

Try to see what is the status of the new file.
Do you understand the status?
Then try to change its status to be able to commit it:
- first add it to the staging area. This is done with the `git add` command. What do you observe? Can you find the information stating that your file is ready to be committed (i.e. in the staging area)?
- commit the content of the staging area with the `git commit` command: `git commit -m "my first commit message"` where you can see that a message is directly attached to the commit with the `-m` option.

Once this is done, check the status.
Did you expect such output?

Note that you can use the `*` to add several files.
For example to add all the Python script files (the files with the `py` extension) you can use: 
```
$ git add *.py
```
You can also add a whole directory and it's content if needed:
```
$ git add my_directory
```

# Exercise 3: modifying files

Edit the file with your name in it (or any other committed file).
For example you can add the workshop's date next to your name:
```
Alfred Escher 2020.07.18
```
or remove your last name:
```
Alfred
```

Check the status. 
What do you see?
Do you understand the output?

Add the modified file to the staging area.

While files are in the staging area, Git can show you all the modifications that are going to be committed with the `git diff` command.

If you want to see them for a specific file you can add the file name after the `git diff` command:
```
$ git diff --staged alfred.escher.txt
```

Now commit (with a message) your you modifications to the repository.
You can check if everything went well by checking the status of the repository and the log with `git log`.

# Exercise 4: move or rename files

## Moving files

Create a subdirectory (for example `users`) in your working directory and move a committed file to this new subdirectory with the `git mv` command.
Note that you can also move several files or wholes directories at once, like it's done with the `add`.

Try to commit this change and check the status between each step to be sure to understand what is happening.

Now repeat the exercises with the `mv` command (i.e. you moved your file without using git).
You should notice that more steps are required to commit the change.


## Renaming files

Renaming a file is simply a move to the same location with a different name, e.g.:
```
$ git mv alfred.escher.txt jerome.franel.txt
```
Try it, and inspect the status of the file.
Create a commit if you want.

Note that the recommended practice is to use `git mv` isntead of and `mv` followed by a `git add`.

# Exercise 5: deleting files

Try to delete (i.e. remove) a committed file with the `git rm` command,
e.g.:
```
$ git rm alfred.escher.txt
```
and try to commit that change.
Note that, like with the `git add` command you can move several files or wholes directories.

Then try the same exercises with the `rm` command on a committed file, e.g.:
```
$ rm alfred.escher.txt
```

Note that the recommended practice is to use `git rm` isntead of and `rm` followed by a `git add`.

# Reverting mistakes

What happen if you did an horrible mistake? 
Like deleting a super important function from your source, or adding a feature that totally break the code.

No worries! 
This is the focus of the next two exercises.
There are two ways of restoring the file: the `checkout` (exercise 6) and the `reset` (exercise 7) but this depend "where" your files are.

# Exercise 6: checkout of an old commit

If the file is **committed** you should use the `checkout` command to revert to a specific commit.
To do this, you need the commit number.
This one is usually available through the `log` command.
Then you provide to `checkout` the commit number, the `--` and the file you want to checkout, e.g.:
```
$ git checkout cb3aa8 -- alfred.escher.txt
```
as you can see, the whole commit number is not needed.
The first 4 characters should be enough.

What do you expect to see in the status?
Do you understand the output?

What about the content of the file?
Does it correspond to the checkout old commit?

If you want to keep it like it is in the older state, you need to **create a new commit** with that version of the file. Try it.

If you want to checkout a full commit, use the command like that:
```
$ git checkout cb3aa8 .
```
It's very **important that you don't forget the** `.` and that you execute the command at the **root of your working directory**.
The `.` indicate that git should checkout all file in the content directory and all the subdirectories only.
This is why you should be at the root.

To make these change permanent, you should **create a new commit** after the checkout.

Note that there exist other ways to **really revert commits**.
But we only show you the operations you can revert (i.e. the non-destructive ones).

# Exercise 7: reset a file

What can you do if a file should not be part in the commit, but it's nevertheless **in the staging area**?

If you want to remove a file from the staging area, simply use `reset` followed by the file name:
```
$ git reset alfred.escher.txt
```

Add a modified file to the staging area and check the status.
Then try to remove it from the staging area with the `git reset` command and check the status.

Did you see the output stating that your file is not staged anymore?

# Exercise 8: ignoring files or directories

Some files may be private or have no point of being part of the repository.
So we can keep them untracked.
In that case Git will not monitor or manage them.
However having a huge list of untracked files may makes the `git status` output hard to read.

Create three files: `temp_result_01.csv`, `temp_result_02.csv` and `temp_result_03.csv` to emulate temporary and unwanted files in your repository.

Then you should see them as untracked with `git status`.
This should "pollute" your output.

Fortunately Git can use a list of file to ignore.
To do so you need to create a hidden file named `.gitignore` (do not forget the `.` at the begining).
And you can add in the `.gitignore` file the file names to ignore in that file:
```
temp_result_01.csv
temp_result_02.csv
temp_result_03.csv
```

And then you can see that the file are not listed anymore as untracked with `git status`.

And of course the `.gitignore` is a file to manage like any other file in Git.
You should commit it in the repository.

It's possible to use pattern to make the `.gitignore` more readable and easier to fill.
For example we also could have written in the `.gitignore` file:
```
temp_result_*.csv
```
You'll find more information about patterns in the [documentation](https://git-scm.com/docs/gitignore).
