# Working with a remote and GitLab

In the previous case the repository was only local.
But it's common with Git to work with a remote repository.
That is to say, one which is not on the local machine.

For example in the centralized workflow, one can see the remote as the `shared repository`:

![cwf](fig/centralWF.png)

Usually the remote is on a machine which is always accessible:
- not turned off,
- connected to the network.

For example a GitHub repository is (should be always) available as it is the case for HEPIA's GitLab.

Here you'll learn to work with remote repository on GitLab.
We'll study two use cases:
1. You start a new project from scratch on GitLab.
2. You already have an existing project and you want to use Git and GitLab to manage it.

Check first if you can connect to [HEPIA's GitLab](https://githepia.hesge.ch/) using your HEPIA credentials.

Note: depending on the configuration, the default branch may not be master anymore, but main.
If the commands don't work with master, try to use main instead.

# Exercise 1: add your SSH key to GitLab

If you already setup your SSH key with GitLab, you can jump to Exercise 2.

If not, there are two cases:
- you already have an SSH key. Check if you have a file name `id_rsa.pub` in your `.ssh` folder.

- You don't have an SSH key. So you should generate by following this nice [tutorial](https://help.github.com/en/articles/generating-a-new-ssh-key-and-adding-it-to-the-ssh-agent). Follow the instruction in the "Generating a new SSH key" section.
At the end of the process this will generate a file named `.ssh/id_rsa.pub`.

To use your SSH key, add it in your [GitLab user settings](https://githepia.hesge.ch/-/profile/keys):

![glssh](fig/gitlab_ssh.png)

Past the content of `id_rsa.pub` in the **Key** text area and name it by adding a name in the **title** text field. Beware of not adding your private key.
Then click on the **Add key** button.

This is needed in order to be correctly identified by the server, otherwise the server cannot know who you are when you'll use `git`.

# Exercise 2: create a remote GitLab repository

To create a new project (i.e. a new repository), on the top burger menu, go to:
```
projects -> your projects -> View all projects
```

![glporj](fig/gl_proj.png)

Then click on the green button labeled `New project`, then `Create blank project` and fill the requested information.

![glnp](fig/new_proj_gl.png)

That's all!
You created an empty repository. You'll learn to clone it in the next exercise.

# Exercise 3: cloning a remote repository

Try to clone the repository you just created with the `git clone` command. 
To do it you should use its SSH address.
You can find the address here:

![gilabrepo](fig/gitlab_repo_add.png)

and then use it with the clone command.

**Do not clone your remote in the local repository you created in the previous exercises!** This because the working directory already contains a local repository. We'll see how to add a remote in an existing local repository in a next exercise.

For example:
```
$ git clone git@ssh.hesge.ch:my-gitlab-repository.git
```
and the output should states that the repository has been cloned. 
Git will create a directory having the name of the project and will contain a `.git` subdirecory.
This new directory will be your working directory.

If you experience difficulties, do not hesitate to ask one of the teaching assistants.

Once a repository is cloned, you can add, edit and delete files, and also (see the next exercise) you can push and pull updates.

# Exercise 4: synchronizing with the remote

Create at least one commit in your new repository.
From this point your commits only exist in your **local repository**. 
If you want your commits to be available on the remote, you have to synchronized them.

The synchronization can be done in two "directions":
- from the remote to your local machine: you **pull** from the remote to your machine,
- from your local machine to the remote: you **push** to the remote.

![gitflow](fig/git_flow_command.png)

## The PUSH direction
In this case we're interested in **pushing our commit to the remote**.
To do so, you use the command `git push`:
```
$ git push origin master
```
Here `origin` is the name of the remote.
By default it has this name, unless you explicitly change it.

As we're using branches, you're only working with the default and only branch: `master` or `main`.
When you push something you have to specify where it is:
- the name of the remote,
- and the corresponding branch on the remote.

## The PULL direction

Great, now your commits have been pushed to the remote. But how can a fellow developer get them? Or simply how can you get them on a different machines?

Again you have to synchronize your local repository with the remote.
This must be done from the machine where you need to **pull** the remote repository.
To do so, you use the command `git pull`:
```
$ git pull origin master 
```
in that specific case, the output will indicate that both repositories (the local one and the remote) are synchronized.

You can "simulate" having local repositories on different machine by **cloning the remote** repository in **another directory**.

Then you will push and pull from these different directories to see what happen.
For example:
```
$ mkdir machine_a
$ cd machine_a
$ git clone git@ssh.hesge.ch:my-gitlab-project.git
```
then add a new file or modify one of the existing file and commit the modification wihout pushing.

Then repeat this in another directory, i.e.:
```
$ mkdir machine_b
$ cd machine_b
$ git clone git@ssh.hesge.ch:my-gitlab-project.git
```

Do you see the commit?
Does it correspond to what you expected?
Do a `git log` to understand what happened.

Go to `machine_a` and push your commits:
```
$ git push origin master
```
and then go to `machine_b`.
Do you see the commit done from `machine_a`?

Now from `machine_b` pull the remote:
```
$ git pull origin master
```
Do you understand the output?
Do you see now the new commit from `machine_a`?
Do a `git log` to check that everything is fine.

# Exercise 5: working with an existing project

To integrate Git and GitLab with an already existing project or working directory:
- First create a new project on GitLab. Let's assume it is `my-gitlab-project`.
- Then on your machine go to your existing working directory. For example let's say `my-existing-project`.
- Then init the repository (like in the previous exercise) wit `git init` in `my-existing-project`.
- Finally add the remote:
`git remote add origin git@ssh.hesge.ch:my-gitlab-project.git`

You can find the address of the remote like we found the address used with the clone command.

You can now use git to add the files and create a commit that you can push on your remote GitLab repository. All like you did in the previous exercises.
